@@ -17,26 +17,30 @@
 
 package org.apache.kyuubi.engine.flink.operation
 
+import java.nio.file.Files
 import java.sql.DatabaseMetaData
+import java.util.UUID
+
+import scala.collection.JavaConverters._
 
-import org.apache.flink.table.api.EnvironmentSettings.DEFAULT_BUILTIN_CATALOG
-import org.apache.flink.table.api.EnvironmentSettings.DEFAULT_BUILTIN_DATABASE
 import org.apache.flink.table.types.logical.LogicalTypeRoot
-import org.apache.hive.service.rpc.thrift.{TExecuteStatementReq, TFetchResultsReq, TOpenSessionReq}
+import org.apache.hive.service.rpc.thrift._
 import org.scalatest.concurrent.PatienceConfiguration.Timeout
 import org.scalatest.time.SpanSugar._
 
-import org.apache.kyuubi.config.KyuubiConf
+import org.apache.kyuubi.config.KyuubiConf._
 import org.apache.kyuubi.config.KyuubiConf.OperationModes.NONE
+import org.apache.kyuubi.engine.flink.FlinkEngineUtils._
 import org.apache.kyuubi.engine.flink.WithFlinkSQLEngine
 import org.apache.kyuubi.engine.flink.result.Constants
+import org.apache.kyuubi.engine.flink.util.TestUserClassLoaderJar
 import org.apache.kyuubi.operation.HiveJDBCTestHelper
 import org.apache.kyuubi.operation.meta.ResultSetSchemaConstant._
 import org.apache.kyuubi.service.ServiceState._
 
 class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
   override def withKyuubiConf: Map[String, String] =
-    Map(KyuubiConf.OPERATION_PLAN_ONLY_MODE.key -> NONE.toString)
+    Map(OPERATION_PLAN_ONLY_MODE.key -> NONE.toString)
 
   override protected def jdbcUrl: String =
     s"jdbc:hive2://${engine.frontendServices.head.connectionUrl}/;"
@@ -63,6 +67,17 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
     }
   }
 
+  test("get columns") {
+    withJdbcStatement() { statement =>
+      val exceptionMsg = intercept[Exception](statement.getConnection.getMetaData.getColumns(
+        null,
+        null,
+        null,
+        null)).getMessage
+      assert(exceptionMsg.contains(s"Unsupported Operation type GetColumns"))
+    }
+  }
+
   test("get type info") {
     withJdbcStatement() { statement =>
       val typeInfo = statement.getConnection.getMetaData.getTypeInfo
@@ -335,16 +350,18 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
     withJdbcStatement() { statement =>
       val metaData = statement.getConnection.getMetaData
       var resultSet = metaData.getSchemas(null, null)
+      val defaultCatalog = "default_catalog"
+      val defaultDatabase = "default_database"
       while (resultSet.next()) {
-        assert(resultSet.getString(TABLE_SCHEM) === DEFAULT_BUILTIN_DATABASE)
-        assert(resultSet.getString(TABLE_CATALOG) === DEFAULT_BUILTIN_CATALOG)
+        assert(resultSet.getString(TABLE_SCHEM) === defaultDatabase)
+        assert(resultSet.getString(TABLE_CATALOG) === defaultCatalog)
       }
       resultSet = metaData.getSchemas(
-        DEFAULT_BUILTIN_CATALOG.split("_").apply(0),
-        DEFAULT_BUILTIN_DATABASE.split("_").apply(0))
+        defaultCatalog.split("_").apply(0),
+        defaultDatabase.split("_").apply(0))
       while (resultSet.next()) {
-        assert(resultSet.getString(TABLE_SCHEM) === DEFAULT_BUILTIN_DATABASE)
-        assert(resultSet.getString(TABLE_CATALOG) === DEFAULT_BUILTIN_CATALOG)
+        assert(resultSet.getString(TABLE_SCHEM) === defaultDatabase)
+        assert(resultSet.getString(TABLE_CATALOG) === defaultCatalog)
       }
     }
   }
@@ -559,7 +576,14 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
       val metaData = resultSet.getMetaData
       assert(metaData.getColumnType(1) === java.sql.Types.ARRAY)
       assert(resultSet.next())
-      assert(resultSet.getObject(1).toString == "[\"v1\",\"v2\",\"v3\"]")
+      if (isFlinkVersionEqualTo("1.14")) {
+        val expected = """["v1","v2","v3"]"""
+        assert(resultSet.getObject(1).toString == expected)
+      }
+      if (isFlinkVersionAtLeast("1.15")) {
+        val expected = "[v1,v2,v3]"
+        assert(resultSet.getObject(1).toString == expected)
+      }
     }
   }
 
@@ -579,8 +603,14 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
       val resultSet =
         statement.executeQuery("select (1, '2', true)")
       assert(resultSet.next())
-      assert(
-        resultSet.getString(1) == "{INT NOT NULL:1,CHAR(1) NOT NULL:\"2\",BOOLEAN NOT NULL:true}")
+      if (isFlinkVersionEqualTo("1.14")) {
+        val expected = """{INT NOT NULL:1,CHAR(1) NOT NULL:"2",BOOLEAN NOT NULL:true}"""
+        assert(resultSet.getString(1) == expected)
+      }
+      if (isFlinkVersionAtLeast("1.15")) {
+        val expected = """{INT NOT NULL:1,CHAR(1) NOT NULL:2,BOOLEAN NOT NULL:true}"""
+        assert(resultSet.getString(1) == expected)
+      }
       val metaData = resultSet.getMetaData
       assert(metaData.getColumnType(1) === java.sql.Types.STRUCT)
     }
@@ -590,8 +620,13 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
     withJdbcStatement() { statement =>
       val resultSet = statement.executeQuery("select encode('kyuubi', 'UTF-8')")
       assert(resultSet.next())
-      assert(
-        resultSet.getString(1) == "kyuubi")
+      if (isFlinkVersionEqualTo("1.14")) {
+        assert(resultSet.getString(1) == "kyuubi")
+      }
+      if (isFlinkVersionAtLeast("1.15")) {
+        // TODO: validate table results after FLINK-28882 is resolved
+        assert(resultSet.getString(1) == "k")
+      }
       val metaData = resultSet.getMetaData
       assert(metaData.getColumnType(1) === java.sql.Types.BINARY)
     }
@@ -606,6 +641,16 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
     })
   }
 
+  test("execute statement - select count") {
+    withJdbcStatement() { statement =>
+      statement.execute(
+        "create table tbl_src (a int) with ('connector' = 'datagen', 'number-of-rows' = '100')")
+      val resultSet = statement.executeQuery(s"select count(a) from tbl_src")
+      assert(resultSet.next())
+      assert(resultSet.getInt(1) === 100)
+    }
+  }
+
   test("execute statement - show functions") {
     withJdbcStatement() { statement =>
       val resultSet = statement.executeQuery("show functions")
@@ -651,6 +696,22 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
     })
   }
 
+  test("execute statement - set/get catalog") {
+    withSessionConf()(
+      Map(ENGINE_OPERATION_CONVERT_CATALOG_DATABASE_ENABLED.key -> "true"))(
+      Map.empty) {
+      withJdbcStatement() { statement =>
+        statement.executeQuery("create catalog cat_a with ('type'='generic_in_memory')")
+        val catalog = statement.getConnection.getCatalog
+        assert(catalog == "default_catalog")
+        statement.getConnection.setCatalog("cat_a")
+        val changedCatalog = statement.getConnection.getCatalog
+        assert(changedCatalog == "cat_a")
+        assert(statement.execute("drop catalog cat_a"))
+      }
+    }
+  }
+
   test("execute statement - create/alter/drop database") {
     // TODO: validate table results after FLINK-25558 is resolved
     withJdbcStatement()({ statement =>
@@ -660,10 +721,26 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
     })
   }
 
+  test("execute statement - set/get database") {
+    withSessionConf()(
+      Map(ENGINE_OPERATION_CONVERT_CATALOG_DATABASE_ENABLED.key -> "true"))(
+      Map.empty) {
+      withJdbcStatement()({ statement =>
+        statement.executeQuery("create database db_a")
+        val schema = statement.getConnection.getSchema
+        assert(schema == "default_database")
+        statement.getConnection.setSchema("db_a")
+        val changedSchema = statement.getConnection.getSchema
+        assert(changedSchema == "db_a")
+        assert(statement.execute("drop database db_a"))
+      })
+    }
+  }
+
   test("execute statement - create/alter/drop table") {
     // TODO: validate table results after FLINK-25558 is resolved
     withJdbcStatement()({ statement =>
-      statement.executeQuery("create table tbl_a (a string)")
+      statement.executeQuery("create table tbl_a (a string) with ('connector' = 'blackhole')")
       assert(statement.execute("alter table tbl_a rename to tbl_b"))
       assert(statement.execute("drop table tbl_b"))
     })
@@ -763,7 +840,7 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
   }
 
   test("ensure result max rows") {
-    withSessionConf()(Map(KyuubiConf.ENGINE_FLINK_MAX_ROWS.key -> "200"))(Map.empty) {
+    withSessionConf()(Map(ENGINE_FLINK_MAX_ROWS.key -> "200"))(Map.empty) {
       withJdbcStatement() { statement =>
         statement.execute("create table tbl_src (a bigint) with ('connector' = 'datagen')")
         val resultSet = statement.executeQuery(s"select a from tbl_src")
@@ -775,4 +852,68 @@ class FlinkOperationSuite extends WithFlinkSQLEngine with HiveJDBCTestHelper {
       }
     }
   }
+
+  test("execute statement - add/remove/show jar") {
+    val jarName = s"newly-added-${UUID.randomUUID()}.jar"
+    val newJar = TestUserClassLoaderJar.createJarFile(
+      Files.createTempDirectory("add-jar-test").toFile,
+      jarName,
+      GENERATED_UDF_CLASS,
+      GENERATED_UDF_CODE).toPath
+
+    withMultipleConnectionJdbcStatement()({ statement =>
+      statement.execute(s"add jar '$newJar'")
+
+      val showJarsResultAdded = statement.executeQuery("show jars")
+      var exists = false
+      while (showJarsResultAdded.next()) {
+        if (showJarsResultAdded.getString(1).contains(jarName)) {
+          exists = true
+        }
+      }
+      assert(exists)
+
+      statement.execute(s"remove jar '$newJar'")
+      val showJarsResultRemoved = statement.executeQuery("show jars")
+      exists = false
+      while (showJarsResultRemoved.next()) {
+        if (showJarsResultRemoved.getString(1).contains(jarName)) {
+          exists = true
+        }
+      }
+      assert(!exists)
+    })
+  }
+
+  test("set session conf - default database") {
+    def assertDefaultDatabase(
+        client: TCLIService.Iface,
+        database: String,
+        expectSuccess: Boolean): Unit = {
+      val req = new TOpenSessionReq()
+      req.setUsername("kyuubi")
+      req.setPassword("anonymous")
+      val conf = Map("use:database" -> database)
+      req.setConfiguration(conf.asJava)
+      val tOpenSessionResp = client.OpenSession(req)
+      val status = tOpenSessionResp.getStatus
+      if (expectSuccess) {
+        assert(status.getStatusCode === TStatusCode.SUCCESS_STATUS)
+      } else {
+        assert(status.getStatusCode === TStatusCode.ERROR_STATUS)
+        assert(status.getErrorMessage.contains(
+          s"A database with name [$database] does not exist"))
+      }
+    }
+
+    withThriftClient { client =>
+      assertDefaultDatabase(client, "default", true)
+    }
+    withThriftClient { client =>
+      assertDefaultDatabase(client, "default2", false)
+    }
+    withThriftClient { client =>
+      assertDefaultDatabase(client, "default_database", true)
+    }
+  }
 }
