@@ -18,8 +18,10 @@
 package org.apache.kyuubi.engine.spark
 
 import java.time.Instant
-import java.util.concurrent.CountDownLatch
+import java.util.concurrent.{CountDownLatch, ScheduledExecutorService, TimeUnit}
+import java.util.concurrent.atomic.AtomicBoolean
 
+import scala.concurrent.duration.Duration
 import scala.util.control.NonFatal
 
 import org.apache.spark.{ui, SparkConf}
@@ -29,22 +31,26 @@ import org.apache.spark.sql.SparkSession
 
 import org.apache.kyuubi.{KyuubiException, Logging, Utils}
 import org.apache.kyuubi.Utils._
-import org.apache.kyuubi.config.KyuubiConf
+import org.apache.kyuubi.config.{KyuubiConf, KyuubiReservedKeys}
 import org.apache.kyuubi.config.KyuubiConf._
 import org.apache.kyuubi.config.KyuubiReservedKeys.KYUUBI_ENGINE_SUBMIT_TIME_KEY
 import org.apache.kyuubi.engine.spark.SparkSQLEngine.{countDownLatch, currentEngine}
-import org.apache.kyuubi.engine.spark.events.{EngineEvent, EngineEventsStore, SparkEventLoggingService}
-import org.apache.kyuubi.events.EventLogging
+import org.apache.kyuubi.engine.spark.events.{EngineEvent, EngineEventsStore, SparkEventHandlerRegister}
+import org.apache.kyuubi.events.EventBus
 import org.apache.kyuubi.ha.HighAvailabilityConf._
 import org.apache.kyuubi.ha.client.RetryPolicies
 import org.apache.kyuubi.service.Serverable
-import org.apache.kyuubi.util.SignalRegister
+import org.apache.kyuubi.util.{SignalRegister, ThreadUtils}
 
 case class SparkSQLEngine(spark: SparkSession) extends Serverable("SparkSQLEngine") {
 
   override val backendService = new SparkSQLBackendService(spark)
   override val frontendServices = Seq(new SparkTBinaryFrontendService(this))
 
+  private val shutdown = new AtomicBoolean(false)
+
+  @volatile private var lifetimeTerminatingChecker: Option[ScheduledExecutorService] = None
+
   override def initialize(conf: KyuubiConf): Unit = {
     val listener = new SparkSQLEngineListener(this)
     spark.sparkContext.addSparkListener(listener)
@@ -62,11 +68,56 @@ case class SparkSQLEngine(spark: SparkSession) extends Serverable("SparkSQLEngin
       assert(currentEngine.isDefined)
       currentEngine.get.stop()
     })
+
+    startLifetimeTerminatingChecker(() => {
+      assert(currentEngine.isDefined)
+      currentEngine.get.stop()
+    })
+  }
+
+  override def stop(): Unit = if (shutdown.compareAndSet(false, true)) {
+    super.stop()
+    lifetimeTerminatingChecker.foreach(checker => {
+      val shutdownTimeout = conf.get(ENGINE_EXEC_POOL_SHUTDOWN_TIMEOUT)
+      ThreadUtils.shutdown(
+        checker,
+        Duration(shutdownTimeout, TimeUnit.MILLISECONDS))
+    })
   }
 
   override protected def stopServer(): Unit = {
     countDownLatch.countDown()
   }
+
+  private[kyuubi] def startLifetimeTerminatingChecker(stop: () => Unit): Unit = {
+    val interval = conf.get(ENGINE_CHECK_INTERVAL)
+    val maxLifetime = conf.get(ENGINE_SPARK_MAX_LIFETIME)
+    val deregistered = new AtomicBoolean(false)
+    if (maxLifetime > 0) {
+      val checkTask: Runnable = () => {
+        if (!shutdown.get && System.currentTimeMillis() - getStartTime > maxLifetime) {
+          if (deregistered.compareAndSet(false, true)) {
+            info(s"Spark engine has been running for more than $maxLifetime ms," +
+              s" deregistering from engine discovery space.")
+            frontendServices.flatMap(_.discoveryService).foreach(_.stop())
+          }
+
+          if (backendService.sessionManager.getOpenSessionCount <= 0) {
+            info(s"Spark engine has been running for more than $maxLifetime ms" +
+              s" and no open session now, terminating")
+            stop()
+          }
+        }
+      }
+      lifetimeTerminatingChecker =
+        Some(ThreadUtils.newDaemonSingleThreadScheduledExecutor("spark-engine-lifetime-checker"))
+      lifetimeTerminatingChecker.get.scheduleWithFixedDelay(
+        checkTask,
+        interval,
+        interval,
+        TimeUnit.MILLISECONDS)
+    }
+  }
 }
 
 object SparkSQLEngine extends Logging {
@@ -83,6 +134,8 @@ object SparkSQLEngine extends Logging {
 
   private val countDownLatch = new CountDownLatch(1)
 
+  private val sparkSessionCreated = new AtomicBoolean(false)
+
   SignalRegister.registerLogger(logger)
   setupConf()
 
@@ -95,6 +148,10 @@ object SparkSQLEngine extends Logging {
     _sparkConf.setIfMissing("spark.sql.legacy.castComplexTypesToString.enabled", "true")
     _sparkConf.setIfMissing("spark.master", "local")
     _sparkConf.setIfMissing("spark.ui.port", "0")
+    _sparkConf.set(
+      "spark.redaction.regex",
+      _sparkConf.get("spark.redaction.regex", "(?i)secret|password|token|access[.]key")
+        + "|zookeeper.auth.digest")
     // register the repl's output dir with the file server.
     // see also `spark.repl.classdir`
     _sparkConf.set("spark.repl.class.outputDir", outputDir.toFile.getAbsolutePath)
@@ -124,37 +181,37 @@ object SparkSQLEngine extends Logging {
   }
 
   def createSpark(): SparkSession = {
+    val engineCredentials = kyuubiConf.getOption(KyuubiReservedKeys.KYUUBI_ENGINE_CREDENTIALS_KEY)
+    kyuubiConf.unset(KyuubiReservedKeys.KYUUBI_ENGINE_CREDENTIALS_KEY)
+    _sparkConf.remove(s"spark.${KyuubiReservedKeys.KYUUBI_ENGINE_CREDENTIALS_KEY}")
+
     val session = SparkSession.builder.config(_sparkConf).getOrCreate
-    (kyuubiConf.get(ENGINE_INITIALIZE_SQL) ++ kyuubiConf.get(ENGINE_SESSION_INITIALIZE_SQL))
-      .foreach { sqlStr =>
-        session.sparkContext.setJobGroup(
-          "engine_initializing_queries",
-          sqlStr,
-          interruptOnCancel = true)
-        debug(s"Execute session initializing sql: $sqlStr")
-        session.sql(sqlStr).isEmpty
-        session.sparkContext.clearJobGroup()
-      }
+
+    engineCredentials.filter(_.nonEmpty).foreach { credentials =>
+      SparkTBinaryFrontendService.renewDelegationToken(session.sparkContext, credentials)
+    }
+
+    KyuubiSparkUtil.initializeSparkSession(
+      session,
+      kyuubiConf.get(ENGINE_INITIALIZE_SQL) ++ kyuubiConf.get(ENGINE_SESSION_INITIALIZE_SQL))
     session
   }
 
   def startEngine(spark: SparkSession): Unit = {
     currentEngine = Some(new SparkSQLEngine(spark))
     currentEngine.foreach { engine =>
-      // start event logging ahead so that we can capture all statuses
-      val eventLogging = new SparkEventLoggingService(spark.sparkContext)
       try {
-        eventLogging.initialize(kyuubiConf)
-        eventLogging.start()
+        // start event logging ahead so that we can capture all statuses
+        initLoggerEventHandler(kyuubiConf)
       } catch {
         case NonFatal(e) =>
-          // Don't block the main process if the `EventLoggingService` failed to start
-          warn(s"Failed to initialize EventLoggingService: ${e.getMessage}", e)
+          // Don't block the main process if the `LoggerEventHandler` failed to start
+          warn(s"Failed to initialize LoggerEventHandler: ${e.getMessage}", e)
       }
 
       try {
         engine.initialize(kyuubiConf)
-        EventLogging.onEvent(EngineEvent(engine))
+        EventBus.post(EngineEvent(engine))
       } catch {
         case t: Throwable =>
           throw new KyuubiException(s"Failed to initialize SparkSQLEngine: ${t.getMessage}", t)
@@ -170,14 +227,18 @@ object SparkSQLEngine extends Logging {
           kyuubiConf)
         val event = EngineEvent(engine)
         info(event)
-        EventLogging.onEvent(event)
+        EventBus.post(event)
       } catch {
         case t: Throwable =>
           throw new KyuubiException(s"Failed to start SparkSQLEngine: ${t.getMessage}", t)
       }
       // Stop engine before SparkContext stopped to avoid calling a stopped SparkContext
       addShutdownHook(() => engine.stop(), SPARK_CONTEXT_SHUTDOWN_PRIORITY + 2)
-      addShutdownHook(() => eventLogging.stop(), SPARK_CONTEXT_SHUTDOWN_PRIORITY + 1)
+    }
+
+    def initLoggerEventHandler(conf: KyuubiConf): Unit = {
+      val sparkEventRegister = new SparkEventHandlerRegister(spark)
+      sparkEventRegister.registerEngineEventLoggers(conf)
     }
   }
 
@@ -191,12 +252,14 @@ object SparkSQLEngine extends Logging {
     val totalInitTime = startedTime - submitTime
     if (totalInitTime > initTimeout) {
       throw new KyuubiException(s"The total engine initialization time ($totalInitTime ms)" +
-        s" exceeds `kyuubi.session.engine.initialize.timeout` ($initTimeout ms)," +
+        s" exceeds ${ENGINE_INIT_TIMEOUT.key} ($initTimeout ms)," +
         s" and submitted at $submitTime.")
     } else {
       var spark: SparkSession = null
       try {
+        startInitTimeoutChecker(submitTime, initTimeout)
         spark = createSpark()
+        sparkSessionCreated.set(true)
         try {
           startEngine(spark)
           // blocking main thread
@@ -205,14 +268,21 @@ object SparkSQLEngine extends Logging {
           case e: KyuubiException => currentEngine match {
               case Some(engine) =>
                 engine.stop()
-                val event = EngineEvent(engine).copy(diagnostic = e.getMessage)
-                EventLogging.onEvent(event)
+                val event = EngineEvent(engine)
+                  .copy(endTime = System.currentTimeMillis(), diagnostic = e.getMessage)
+                EventBus.post(event)
                 error(event, e)
               case _ => error("Current SparkSQLEngine is not created.")
             }
 
         }
       } catch {
+        case i: InterruptedException if !sparkSessionCreated.get =>
+          error(
+            s"The Engine main thread was interrupted, possibly due to `createSpark` timeout." +
+              s" The `kyuubi.session.engine.initialize.timeout` is ($initTimeout ms) " +
+              s" and submitted at $submitTime.",
+            i)
         case t: Throwable => error(s"Failed to instantiate SparkSession: ${t.getMessage}", t)
       } finally {
         if (spark != null) {
@@ -221,4 +291,18 @@ object SparkSQLEngine extends Logging {
       }
     }
   }
+
+  private def startInitTimeoutChecker(startTime: Long, timeout: Long): Unit = {
+    val mainThread = Thread.currentThread()
+    new Thread(
+      () => {
+        while (System.currentTimeMillis() - startTime < timeout && !sparkSessionCreated.get()) {
+          Thread.sleep(500)
+        }
+        if (!sparkSessionCreated.get()) {
+          mainThread.interrupt()
+        }
+      },
+      "CreateSparkTimeoutChecker").start()
+  }
 }
