@@ -18,11 +18,13 @@
 package org.apache.kyuubi.engine.spark.operation
 
 import org.apache.spark.sql.Row
+import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types.StructType
 
+import org.apache.kyuubi.KyuubiSQLException
 import org.apache.kyuubi.config.KyuubiConf.OPERATION_PLAN_ONLY_EXCLUDES
 import org.apache.kyuubi.config.KyuubiConf.OperationModes._
-import org.apache.kyuubi.operation.{ArrayFetchIterator, IterableFetchIterator, OperationType}
+import org.apache.kyuubi.operation.{ArrayFetchIterator, IterableFetchIterator}
 import org.apache.kyuubi.operation.log.OperationLog
 import org.apache.kyuubi.session.Session
 
@@ -33,7 +35,7 @@ class PlanOnlyStatement(
     session: Session,
     override val statement: String,
     mode: OperationMode)
-  extends SparkOperation(OperationType.EXECUTE_STATEMENT, session) {
+  extends SparkOperation(session) {
 
   private val operationLog: OperationLog = OperationLog.createOperationLog(session, getHandle)
   private val planExcludes: Seq[String] = {
@@ -50,32 +52,37 @@ class PlanOnlyStatement(
     } else result.schema
   }
 
-  override protected def runInternal(): Unit = {
+  override protected def runInternal(): Unit = withLocalProperties {
     try {
-      val parsed = spark.sessionState.sqlParser.parsePlan(statement)
-      parsed match {
-        case cmd if planExcludes.contains(cmd.getClass.getSimpleName) =>
-          result = spark.sql(statement)
-          iter = new ArrayFetchIterator(result.collect())
-        case plan => mode match {
-            case PARSE =>
-              iter = new IterableFetchIterator(Seq(Row(plan.toString())))
-            case ANALYZE =>
-              val analyzed = spark.sessionState.analyzer.execute(plan)
-              spark.sessionState.analyzer.checkAnalysis(analyzed)
-              iter = new IterableFetchIterator(Seq(Row(analyzed.toString())))
-            case OPTIMIZE =>
-              val analyzed = spark.sessionState.analyzer.execute(plan)
-              spark.sessionState.analyzer.checkAnalysis(analyzed)
-              val optimized = spark.sessionState.optimizer.execute(analyzed)
-              iter = new IterableFetchIterator(Seq(Row(optimized.toString())))
-            case PHYSICAL =>
-              val physical = spark.sql(statement).queryExecution.sparkPlan
-              iter = new IterableFetchIterator(Seq(Row(physical.toString())))
-            case EXECUTION =>
-              val executed = spark.sql(statement).queryExecution.executedPlan
-              iter = new IterableFetchIterator(Seq(Row(executed.toString())))
-          }
+      SQLConf.withExistingConf(spark.sessionState.conf) {
+        val parsed = spark.sessionState.sqlParser.parsePlan(statement)
+        parsed match {
+          case cmd if planExcludes.contains(cmd.getClass.getSimpleName) =>
+            result = spark.sql(statement)
+            iter = new ArrayFetchIterator(result.collect())
+          case plan => mode match {
+              case PARSE =>
+                iter = new IterableFetchIterator(Seq(Row(plan.toString())))
+              case ANALYZE =>
+                val analyzed = spark.sessionState.analyzer.execute(plan)
+                spark.sessionState.analyzer.checkAnalysis(analyzed)
+                iter = new IterableFetchIterator(Seq(Row(analyzed.toString())))
+              case OPTIMIZE =>
+                val analyzed = spark.sessionState.analyzer.execute(plan)
+                spark.sessionState.analyzer.checkAnalysis(analyzed)
+                val optimized = spark.sessionState.optimizer.execute(analyzed)
+                iter = new IterableFetchIterator(Seq(Row(optimized.toString())))
+              case PHYSICAL =>
+                val physical = spark.sql(statement).queryExecution.sparkPlan
+                iter = new IterableFetchIterator(Seq(Row(physical.toString())))
+              case EXECUTION =>
+                val executed = spark.sql(statement).queryExecution.executedPlan
+                iter = new IterableFetchIterator(Seq(Row(executed.toString())))
+              case UNKNOWN =>
+                throw KyuubiSQLException(s"The operation mode $mode" +
+                  " doesn't support in Spark SQL engine.")
+            }
+        }
       }
     } catch {
       onError()
