@@ -31,6 +31,7 @@ import org.apache.spark.sql.catalyst.analysis.FunctionRegistry
 import org.apache.spark.sql.types._
 
 import org.apache.kyuubi.engine.spark.WithSparkSQLEngine
+import org.apache.kyuubi.engine.spark.schema.SchemaHelper.TIMESTAMP_NTZ
 import org.apache.kyuubi.engine.spark.shim.SparkCatalogShim
 import org.apache.kyuubi.operation.{HiveMetadataTests, SparkQueryTests}
 import org.apache.kyuubi.operation.meta.ResultSetSchemaConstant._
@@ -56,7 +57,7 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
 
   test("get columns operation") {
     val tableName = "spark_get_col_operation"
-    val schema = new StructType()
+    var schema = new StructType()
       .add("c0", "boolean", nullable = false, "0")
       .add("c1", "tinyint", nullable = true, "1")
       .add("c2", "smallint", nullable = false, "2")
@@ -76,6 +77,18 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
       .add("c16", "binary", nullable = false, "16")
       .add("c17", "struct<X: string>", nullable = true, "17")
 
+    // since spark3.3.0
+    if (SPARK_ENGINE_MAJOR_MINOR_VERSION._1 > 3 ||
+      (SPARK_ENGINE_MAJOR_MINOR_VERSION._1 == 3 && SPARK_ENGINE_MAJOR_MINOR_VERSION._2 >= 3)) {
+      schema = schema.add("c18", "interval day", nullable = true, "18")
+        .add("c19", "interval year", nullable = true, "19")
+    }
+    // since spark3.4.0
+    if (SPARK_ENGINE_MAJOR_MINOR_VERSION._1 > 3 ||
+      (SPARK_ENGINE_MAJOR_MINOR_VERSION._1 == 3 && SPARK_ENGINE_MAJOR_MINOR_VERSION._2 >= 4)) {
+      schema = schema.add("c20", "timestamp_ntz", nullable = true, "20")
+    }
+
     val ddl =
       s"""
          |CREATE TABLE IF NOT EXISTS $defaultSchema.$tableName (
@@ -110,7 +123,10 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
           TIMESTAMP,
           STRUCT,
           BINARY,
-          STRUCT)
+          STRUCT,
+          OTHER,
+          OTHER,
+          TIMESTAMP)
 
         var pos = 0
 
@@ -137,6 +153,8 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
             case FloatType => assert(decimalDigits === 7)
             case DoubleType => assert(decimalDigits === 15)
             case TimestampType => assert(decimalDigits === 6)
+            case ntz if ntz.getClass.getSimpleName.equals(TIMESTAMP_NTZ) =>
+              assert(decimalDigits === 6)
             case _ => assert(decimalDigits === 0) // nulls
           }
 
@@ -154,7 +172,7 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
           pos += 1
         }
 
-        assert(pos === 18, "all columns should have been verified")
+        assert(pos === schema.length, "all columns should have been verified")
       }
 
       val rowSet = metaData.getColumns(null, "*", "not_exist", "not_exist")
@@ -431,7 +449,7 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
       val tOpenSessionResp = client.OpenSession(req)
       val status = tOpenSessionResp.getStatus
       assert(status.getStatusCode === TStatusCode.ERROR_STATUS)
-      assert(status.getErrorMessage.contains("Database 'default2' does not exist"))
+      assert(status.getErrorMessage.contains("Database 'default2' not found"))
     }
   }
 
@@ -457,8 +475,7 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
       tFetchResultsReq.setMaxRows(1000)
       val tFetchResultsResp = client.FetchResults(tFetchResultsReq)
       assert(tFetchResultsResp.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
-      assert(tFetchResultsResp.getStatus.getErrorMessage startsWith "Invalid OperationHandle" +
-        " [type=EXECUTE_STATEMENT, identifier:")
+      assert(tFetchResultsResp.getStatus.getErrorMessage startsWith "Invalid OperationHandle [")
 
       val tCloseSessionReq = new TCloseSessionReq()
       tCloseSessionReq.setSessionHandle(tOpenSessionResp.getSessionHandle)
