@@ -22,7 +22,8 @@ import scala.collection.JavaConverters._
 import org.apache.hive.service.rpc.thrift.{TGetInfoType, TGetInfoValue, TProtocolVersion, TRowSet, TTableSchema}
 
 import org.apache.kyuubi.{KyuubiSQLException, Logging}
-import org.apache.kyuubi.events.KyuubiEvent
+import org.apache.kyuubi.config.KyuubiConf
+import org.apache.kyuubi.config.KyuubiReservedKeys.KYUUBI_CLIENT_IP_KEY
 import org.apache.kyuubi.operation.{Operation, OperationHandle}
 import org.apache.kyuubi.operation.FetchOrientation.FetchOrientation
 import org.apache.kyuubi.operation.log.OperationLog
@@ -34,8 +35,10 @@ abstract class AbstractSession(
     val ipAddress: String,
     val conf: Map[String, String],
     val sessionManager: SessionManager) extends Session with Logging {
+  override val handle: SessionHandle = SessionHandle()
 
-  protected def logSessionInfo(msg: String): Unit = info(s"[$user:$ipAddress] $handle - $msg")
+  def clientIpAddress: String = conf.getOrElse(KYUUBI_CLIENT_IP_KEY, ipAddress)
+  protected def logSessionInfo(msg: String): Unit = info(s"[$user:$clientIpAddress] $handle - $msg")
 
   final private val _createTime: Long = System.currentTimeMillis()
   override def createTime: Long = _createTime
@@ -52,6 +55,8 @@ abstract class AbstractSession(
 
   val normalizedConf: Map[String, String] = sessionManager.validateAndNormalizeConf(conf)
 
+  override lazy val name: Option[String] = normalizedConf.get(KyuubiConf.SESSION_NAME.key)
+
   final private val opHandleSet = new java.util.HashSet[OperationHandle]
 
   private def acquire(userAccess: Boolean): Unit = synchronized {
@@ -174,6 +179,40 @@ abstract class AbstractSession(
     runOperation(operation)
   }
 
+  override def getPrimaryKeys(
+      catalogName: String,
+      schemaName: String,
+      tableName: String): OperationHandle = {
+    val operation = sessionManager.operationManager
+      .newGetPrimaryKeysOperation(this, catalogName, schemaName, tableName)
+    runOperation(operation)
+  }
+
+  override def getCrossReference(
+      primaryCatalog: String,
+      primarySchema: String,
+      primaryTable: String,
+      foreignCatalog: String,
+      foreignSchema: String,
+      foreignTable: String): OperationHandle = {
+    val operation = sessionManager.operationManager
+      .newGetCrossReferenceOperation(
+        this,
+        primaryCatalog,
+        primarySchema,
+        primaryTable,
+        foreignCatalog,
+        foreignSchema,
+        foreignTable)
+    runOperation(operation)
+  }
+
+  override def getQueryId(operationHandle: OperationHandle): String = {
+    val operation = sessionManager.operationManager.getOperation(operationHandle)
+    val queryId = sessionManager.operationManager.getQueryId(operation)
+    queryId
+  }
+
   override def cancelOperation(operationHandle: OperationHandle): Unit = withAcquireRelease() {
     sessionManager.operationManager.cancelOperation(operationHandle)
   }
@@ -200,7 +239,7 @@ abstract class AbstractSession(
     }
   }
 
-  override def closeExpiredOperations: Unit = {
+  override def closeExpiredOperations(): Unit = {
     val operations = sessionManager.operationManager
       .removeExpiredOperations(opHandleSet.asScala.toSeq)
     operations.foreach { op =>
@@ -219,6 +258,4 @@ abstract class AbstractSession(
   override def open(): Unit = {
     OperationLog.createOperationLogRootDirectory(this)
   }
-
-  override def getSessionEvent: Option[KyuubiEvent] = None
 }
