@@ -21,8 +21,8 @@ import java.sql.{DatabaseMetaData, ResultSet, SQLException, SQLFeatureNotSupport
 
 import scala.util.Random
 
-import org.apache.kyuubi.KYUUBI_VERSION
-import org.apache.kyuubi.Utils
+import org.apache.kyuubi.{KYUUBI_VERSION, KyuubiSQLException}
+import org.apache.kyuubi.engine.SemanticVersion
 import org.apache.kyuubi.operation.meta.ResultSetSchemaConstant._
 
 // For both `in-memory` and `hive` external catalog
@@ -316,7 +316,6 @@ trait SparkMetadataTests extends HiveJDBCTestHelper {
         () => metaData.storesLowerCaseQuotedIdentifiers(),
         () => metaData.storesMixedCaseIdentifiers(),
         () => metaData.storesMixedCaseQuotedIdentifiers(),
-        () => metaData.getSQLKeywords,
         () => metaData.nullPlusNonNullIsNull,
         () => metaData.supportsConvert,
         () => metaData.supportsTableCorrelationNames,
@@ -397,7 +396,6 @@ trait SparkMetadataTests extends HiveJDBCTestHelper {
         () => metaData.getRowIdLifetime,
         () => metaData.supportsStoredFunctionsUsingCallSyntax,
         () => metaData.autoCommitFailureClosesAllResultSets,
-        () => metaData.getClientInfoProperties,
         () => metaData.getFunctionColumns("", "%", "%", "%"),
         () => metaData.getPseudoColumns("", "%", "%", "%"),
         () => metaData.generatedKeyAlwaysReturned).foreach { func =>
@@ -406,12 +404,14 @@ trait SparkMetadataTests extends HiveJDBCTestHelper {
       }
 
       assert(metaData.allTablesAreSelectable)
+      assert(metaData.getClientInfoProperties.next)
       assert(metaData.getDatabaseProductName === "Apache Kyuubi (Incubating)")
       assert(metaData.getDatabaseProductVersion === KYUUBI_VERSION)
       assert(metaData.getDriverName === "Kyuubi Project Hive JDBC Shaded Client")
       assert(metaData.getDriverVersion === KYUUBI_VERSION)
-      assert(metaData.getDatabaseMajorVersion === Utils.majorVersion(KYUUBI_VERSION))
-      assert(metaData.getDatabaseMinorVersion === Utils.minorVersion(KYUUBI_VERSION))
+      val ver = SemanticVersion(KYUUBI_VERSION)
+      assert(metaData.getDatabaseMajorVersion === ver.majorVersion)
+      assert(metaData.getDatabaseMinorVersion === ver.minorVersion)
       assert(
         metaData.getIdentifierQuoteString === " ",
         "This method returns a space \" \" if identifier quoting is not supported")
@@ -455,10 +455,20 @@ trait SparkMetadataTests extends HiveJDBCTestHelper {
       assert(metaData.getDefaultTransactionIsolation === java.sql.Connection.TRANSACTION_NONE)
       assert(!metaData.supportsTransactions)
       assert(!metaData.getProcedureColumns("", "%", "%", "%").next())
-      intercept[SQLException](metaData.getPrimaryKeys("", "default", ""))
+      try {
+        assert(!metaData.getPrimaryKeys("", "default", "src").next())
+      } catch {
+        case e: Exception =>
+          assert(e.isInstanceOf[SQLException])
+          assert(e.getMessage.contains(KyuubiSQLException.featureNotSupported().getMessage))
+      }
       assert(!metaData.getImportedKeys("", "default", "").next())
-      intercept[SQLException] {
-        metaData.getCrossReference("", "default", "src", "", "default", "src2")
+      try {
+        assert(!metaData.getCrossReference("", "default", "src", "", "default", "src2").next())
+      } catch {
+        case e: Exception =>
+          assert(e.isInstanceOf[SQLException])
+          assert(e.getMessage.contains(KyuubiSQLException.featureNotSupported().getMessage))
       }
       assert(!metaData.getIndexInfo("", "default", "src", true, true).next())
 
