@@ -18,18 +18,23 @@
 package org.apache.kyuubi.operation
 
 import java.sql.{Date, SQLException, SQLTimeoutException, Timestamp}
+import java.util.UUID
 
 import scala.collection.JavaConverters._
 
+import org.apache.commons.io.FileUtils
 import org.apache.commons.lang3.StringUtils
 import org.apache.hive.service.rpc.thrift.{TExecuteStatementReq, TFetchResultsReq, TOpenSessionReq, TStatusCode}
 
 import org.apache.kyuubi.{KYUUBI_VERSION, Utils}
+import org.apache.kyuubi.engine.SemanticVersion
 
 trait SparkQueryTests extends HiveJDBCTestHelper {
 
   protected lazy val SPARK_ENGINE_MAJOR_MINOR_VERSION: (Int, Int) = sparkEngineMajorMinorVersion
 
+  protected lazy val httpMode = false;
+
   test("execute statement - select null") {
     withJdbcStatement() { statement =>
       val resultSet = statement.executeQuery("SELECT NULL AS col")
@@ -174,17 +179,92 @@ trait SparkQueryTests extends HiveJDBCTestHelper {
     }
   }
 
-  test("execute statement - select interval") {
-    // FIXME: [KYUUBI #1250]
-    assume(SPARK_ENGINE_MAJOR_MINOR_VERSION !== ((3, 2)))
+  test("execute statement - select timestamp_ntz") {
+    assume(SPARK_ENGINE_MAJOR_MINOR_VERSION._1 > 3 ||
+      (SPARK_ENGINE_MAJOR_MINOR_VERSION._1 == 3 && SPARK_ENGINE_MAJOR_MINOR_VERSION._2 >= 4))
     withJdbcStatement() { statement =>
-      val resultSet = statement.executeQuery("SELECT interval '1' day AS col")
+      val resultSet = statement.executeQuery(
+        "SELECT make_timestamp_ntz(2022, 03, 24, 18, 08, 31.800) AS col")
       assert(resultSet.next())
-      assert(resultSet.getString("col") === "1 days")
-      assert(resultSet.getMetaData.getColumnType(1) === java.sql.Types.VARCHAR)
+      assert(resultSet.getTimestamp("col") === Timestamp.valueOf("2022-03-24 18:08:31.800"))
       val metaData = resultSet.getMetaData
-      assert(metaData.getPrecision(1) === Int.MaxValue)
-      assert(metaData.getScale(1) === 0)
+      assert(metaData.getColumnType(1) === java.sql.Types.TIMESTAMP)
+      assert(metaData.getPrecision(1) === 29)
+      assert(metaData.getScale(1) === 9)
+    }
+  }
+
+  test("execute statement - select daytime interval") {
+    withJdbcStatement() { statement =>
+      Map(
+        "interval 1 day 1 hour -60 minutes 30 seconds" ->
+          Tuple2("1 00:00:30.000000000", "1 days 30 seconds"),
+        "interval 30 seconds 12345 milliseconds" ->
+          Tuple2("0 00:00:42.345000000", "42.345 seconds"),
+        "interval 1 hour 59 minutes 30 seconds 12345 milliseconds" ->
+          Tuple2("0 01:59:42.345000000", "1 hours 59 minutes 42.345 seconds"),
+        "-interval 2 day" -> Tuple2("-2 00:00:00.000000000", "-2 days"),
+        "interval 59 minutes 30 seconds 12345 milliseconds" ->
+          Tuple2("0 00:59:42.345000000", "59 minutes 42.345 seconds"),
+        "interval 25 hour" -> Tuple2("1 01:00:00.000000000", "25 hours"),
+        "interval 1 hour 62 minutes" -> Tuple2("0 02:02:00.000000000", "2 hours 2 minutes"),
+        "interval 1 day 1 hour 59 minutes 30 seconds 12345 milliseconds" ->
+          Tuple2("1 01:59:42.345000000", "1 days 1 hours 59 minutes 42.345 seconds"),
+        "interval 1 day 1 hour -60 minutes" -> Tuple2("1 00:00:00.000000000", "1 days"),
+        "INTERVAL 30 SECONDS" -> Tuple2("0 00:00:30.000000000", "30 seconds"),
+        "interval -60 minutes 30 seconds" ->
+          Tuple2("-0 00:59:30.000000000", "-59 minutes -30 seconds"),
+        "-interval 200 day" -> Tuple2("-200 00:00:00.000000000", "-200 days"),
+        "interval 1 hour -60 minutes 30 seconds" -> Tuple2("0 00:00:30.000000000", "30 seconds"),
+        "interval 62 minutes" -> Tuple2("0 01:02:00.000000000", "1 hours 2 minutes"),
+        "interval 1 day 1 hour" -> Tuple2("1 01:00:00.000000000", "1 days 1 hours")).foreach {
+        kv => // value -> result pair
+          val resultSet = statement.executeQuery(s"SELECT ${kv._1} AS col")
+          assert(resultSet.next())
+          val result = resultSet.getString("col")
+          val metaData = resultSet.getMetaData
+          if (SPARK_ENGINE_MAJOR_MINOR_VERSION._1 == 3
+            && SPARK_ENGINE_MAJOR_MINOR_VERSION._2 < 2) {
+            // for spark 3.1 and backwards
+            assert(result === kv._2._2)
+            assert(metaData.getPrecision(1) === Int.MaxValue)
+            assert(resultSet.getMetaData.getColumnType(1) === java.sql.Types.VARCHAR)
+          } else {
+            assert(result === kv._2._1)
+            assert(metaData.getPrecision(1) === 29)
+            assert(resultSet.getMetaData.getColumnType(1) === java.sql.Types.OTHER)
+          }
+          assert(metaData.getScale(1) === 0)
+      }
+    }
+  }
+
+  test("execute statement - select year/month interval") {
+    withJdbcStatement() { statement =>
+      Map(
+        "INTERVAL 2022 YEAR" -> Tuple2("2022-0", "2022 years"),
+        "INTERVAL '2021-07' YEAR TO MONTH" -> Tuple2("2021-7", "2021 years 7 months"),
+        "INTERVAL 3 MONTH" -> Tuple2("0-3", "3 months"),
+        "INTERVAL 241 MONTH" -> Tuple2("20-1", "20 years 1 months"),
+        "INTERVAL -1 year -25 MONTH" -> Tuple2("-3-1", "-3 years -1 months"),
+        "INTERVAL 3 year -25 MONTH" -> Tuple2("0-11", "11 months")).foreach { kv =>
+        val resultSet = statement.executeQuery(s"SELECT ${kv._1} AS col")
+        assert(resultSet.next())
+        val result = resultSet.getString("col")
+        val metaData = resultSet.getMetaData
+        if (SPARK_ENGINE_MAJOR_MINOR_VERSION._1 == 3
+          && SPARK_ENGINE_MAJOR_MINOR_VERSION._2 < 2) {
+          // for spark 3.1 and backwards
+          assert(result === kv._2._2)
+          assert(metaData.getPrecision(1) === Int.MaxValue)
+          assert(resultSet.getMetaData.getColumnType(1) === java.sql.Types.VARCHAR)
+        } else {
+          assert(result === kv._2._1)
+          assert(metaData.getPrecision(1) === 11)
+          assert(resultSet.getMetaData.getColumnType(1) === java.sql.Types.OTHER)
+        }
+        assert(metaData.getScale(1) === 0)
+      }
     }
   }
 
@@ -244,17 +324,22 @@ trait SparkQueryTests extends HiveJDBCTestHelper {
 
   test("execute statement - analysis exception") {
     val sql = "select date_sub(date'2011-11-11', '1.2')"
+    val errors = Set(
+      "The second argument of 'date_sub' function needs to be an integer.",
+      // unquoted since Spark-3.4, see https://github.com/apache/spark/pull/36693
+      "The second argument of date_sub function needs to be an integer.")
 
     withJdbcStatement() { statement =>
       val e = intercept[SQLException] {
         statement.executeQuery(sql)
       }
-      assert(e.getMessage
-        .contains("The second argument of 'date_sub' function needs to be an integer."))
+      assert(errors.exists(msg => e.getMessage.contains(msg)))
     }
   }
 
   test("execute statement - select with variable substitution") {
+    assume(!httpMode)
+
     withThriftClient { client =>
       val req = new TOpenSessionReq()
       req.setUsername("chengpan")
@@ -394,7 +479,7 @@ trait SparkQueryTests extends HiveJDBCTestHelper {
         assert(set.getString("plan") startsWith "Create")
         val set0 = statement.executeQuery(setkey)
         assert(set0.next())
-        assert(set0.getString(2) === "optimize")
+        assert(set0.getString(2) === "OPTIMIZE")
         val e1 = intercept[SQLException](statement.executeQuery(dql))
         assert(e1.getMessage.contains("Table or view not found"))
         statement.execute("SET kyuubi.operation.plan.only.mode=analyze")
@@ -420,7 +505,7 @@ trait SparkQueryTests extends HiveJDBCTestHelper {
       rs.next()
       // scala repl will return resX = YYYYY, and here we only check YYYYY
       val sparkVer = rs.getString(1).split("=")(1).trim
-      assert("\\d\\.\\d\\.\\d".r.pattern.matcher(sparkVer).matches())
+      assert("\\d\\.\\d\\.\\d(-SNAPSHOT)?".r.pattern.matcher(sparkVer).matches())
       assert(rs.getMetaData.getColumnName(1) === "output")
     }
   }
@@ -533,6 +618,63 @@ trait SparkQueryTests extends HiveJDBCTestHelper {
     }
   }
 
+  test("scala code with loading external package at runtime ") {
+    val jarDir = Utils.createTempDir().toFile
+
+    withJdbcStatement() { statement =>
+      statement.execute("SET kyuubi.operation.language=scala")
+      val udfCode =
+        """
+          |package test.utils
+          |
+          |object Math {
+          |def add(x: Int, y: Int): Int = x + y
+          |}
+          |
+          |""".stripMargin
+
+      val jarFile = UserJarTestUtils.createJarFile(
+        udfCode,
+        "test",
+        s"test-function-${UUID.randomUUID}.jar",
+        jarDir.toString)
+      val jarBytes = FileUtils.readFileToByteArray(jarFile)
+      val jarStr = new String(java.util.Base64.getEncoder().encode(jarBytes))
+      val jarName = s"test-function-${UUID.randomUUID}.jar"
+
+      val code0 = """spark.sql("SET kyuubi.operation.language").show(false)"""
+      statement.execute(code0)
+
+      // Generate a jar package in spark engine
+      val batchCode =
+        s"""
+           |import java.io.{BufferedOutputStream, File, FileOutputStream}
+           |val dir = spark.sparkContext.getConf.get("spark.repl.class.outputDir")
+           |val jarFile = new File(dir, "$jarName")
+           |val bos = new BufferedOutputStream(new FileOutputStream(jarFile))
+           |val path = "$jarStr"
+           |bos.write(java.util.Base64.getDecoder.decode(path))
+           |bos.close()
+           |val jarPath = jarFile.getAbsolutePath
+           |val fileSize = jarFile.length
+           |""".stripMargin
+      batchCode.split("\n").filter(_.nonEmpty).foreach { code =>
+        val rs = statement.executeQuery(code)
+        rs.next()
+        // scalastyle:off
+        println(rs.getString(1))
+      // scalastyle:on
+      }
+
+      val code1 = s"""spark.sql("add jar " + jarPath)"""
+      val code2 = """val x = test.utils.Math.add(1,2)"""
+      statement.execute(code1)
+      val rs = statement.executeQuery(code2)
+      rs.next()
+      assert(rs.getString(1) == "x: Int = 3")
+    }
+  }
+
   def sparkEngineMajorMinorVersion: (Int, Int) = {
     var sparkRuntimeVer = ""
     withJdbcStatement() { stmt =>
@@ -541,6 +683,7 @@ trait SparkQueryTests extends HiveJDBCTestHelper {
       sparkRuntimeVer = result.getString(1)
       assert(!result.next())
     }
-    Utils.majorMinorVersion(sparkRuntimeVer)
+    val ver = SemanticVersion(sparkRuntimeVer)
+    (ver.majorVersion, ver.minorVersion)
   }
 }
