@@ -18,15 +18,16 @@
 package org.apache.kyuubi.operation.log
 
 import java.io.File
+import java.nio.charset.StandardCharsets
 import java.nio.file.{Files, Paths}
 
 import scala.collection.JavaConverters._
 
-import org.apache.hive.service.rpc.thrift.TProtocolVersion
+import org.apache.hive.service.rpc.thrift.{TProtocolVersion, TRowSet}
 
 import org.apache.kyuubi.{KyuubiFunSuite, KyuubiSQLException, Utils}
 import org.apache.kyuubi.config.KyuubiConf
-import org.apache.kyuubi.operation.{OperationHandle, OperationType}
+import org.apache.kyuubi.operation.OperationHandle
 import org.apache.kyuubi.session.NoopSessionManager
 
 class OperationLogSuite extends KyuubiFunSuite {
@@ -44,9 +45,7 @@ class OperationLogSuite extends KyuubiFunSuite {
       "localhost",
       Map.empty)
     val session = sessionManager.getSession(sHandle)
-    val oHandle = OperationHandle(
-      OperationType.EXECUTE_STATEMENT,
-      TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V10)
+    val oHandle = OperationHandle()
     assert(sessionManager.operationLogRoot.isDefined)
     val operationLogRoot = sessionManager.operationLogRoot.get
 
@@ -97,9 +96,7 @@ class OperationLogSuite extends KyuubiFunSuite {
       "localhost",
       Map.empty)
     val session = sessionManager.getSession(sHandle)
-    val oHandle = OperationHandle(
-      OperationType.EXECUTE_STATEMENT,
-      TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V10)
+    val oHandle = OperationHandle()
 
     OperationLog.createOperationLogRootDirectory(session)
     val operationLog = OperationLog.createOperationLog(session, oHandle)
@@ -148,9 +145,7 @@ class OperationLogSuite extends KyuubiFunSuite {
     assert(logRoot.exists())
     OperationLog.createOperationLogRootDirectory(session)
     assert(logRoot.isFile)
-    val oHandle = OperationHandle(
-      OperationType.EXECUTE_STATEMENT,
-      TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V10)
+    val oHandle = OperationHandle()
     intercept[Exception] {
       val log = OperationLog.createOperationLog(session, oHandle)
       log.read(1)
@@ -171,9 +166,9 @@ class OperationLogSuite extends KyuubiFunSuite {
     val tempDir = Utils.createTempDir().toFile
     tempDir.setExecutable(false)
 
-    sessionManager.setOperationLogRootDir(tempDir.getAbsolutePath + "/operation_logs")
-    assert(sessionManager.operationLogRoot.isDefined)
-    sessionManager.initialize(KyuubiConf())
+    val conf = KyuubiConf()
+      .set(KyuubiConf.SERVER_OPERATION_LOG_DIR_ROOT, tempDir.getAbsolutePath + "/operation_logs")
+    sessionManager.initialize(conf)
     assert(sessionManager.operationLogRoot.isEmpty)
 
     tempDir.setExecutable(true)
@@ -190,9 +185,7 @@ class OperationLogSuite extends KyuubiFunSuite {
       "localhost",
       Map.empty)
     val session = sessionManager.getSession(sHandle)
-    val oHandle = OperationHandle(
-      OperationType.EXECUTE_STATEMENT,
-      TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V10)
+    val oHandle = OperationHandle()
     OperationLog.createOperationLogRootDirectory(session)
 
     val operationLog = OperationLog.createOperationLog(session, oHandle)
@@ -211,4 +204,31 @@ class OperationLogSuite extends KyuubiFunSuite {
     operationLog.close()
     tempDir.toFile.delete()
   }
+
+  test("test seek reader") {
+    val file = Utils.createTempDir().resolve("f")
+    val writer = Files.newBufferedWriter(file, StandardCharsets.UTF_8)
+    try {
+      0.until(10).foreach(x => writer.write(s"$x\n"))
+      writer.flush()
+      writer.close()
+
+      def compareResult(rows: TRowSet, expected: Seq[String]): Unit = {
+        val res = rows.getColumns.get(0).getStringVal.getValues.asScala
+        assert(res.size == expected.size)
+        res.zip(expected).foreach { case (l, r) =>
+          assert(l == r)
+        }
+      }
+
+      val log = new OperationLog(file)
+      compareResult(log.read(-1, 1), Seq("0"))
+      compareResult(log.read(-1, 1), Seq("1"))
+      compareResult(log.read(0, 1), Seq("0"))
+      compareResult(log.read(0, 2), Seq("0", "1"))
+      compareResult(log.read(5, 10), Seq("5", "6", "7", "8", "9"))
+    } finally {
+      Utils.deleteDirectoryRecursively(file.toFile)
+    }
+  }
 }
