@@ -27,8 +27,8 @@ import org.scalatest.time._
 import org.apache.kyuubi.{KyuubiFunSuite, KyuubiSQLException, Utils}
 import org.apache.kyuubi.config.KyuubiConf
 import org.apache.kyuubi.config.KyuubiConf.{FRONTEND_BIND_HOST, FRONTEND_CONNECTION_URL_USE_HOSTNAME, FRONTEND_THRIFT_BINARY_BIND_HOST, FRONTEND_THRIFT_BINARY_BIND_PORT}
-import org.apache.kyuubi.operation.{OperationHandle, OperationType, TClientTestUtils}
-import org.apache.kyuubi.service.TFrontendService.{FeServiceServerContext, SERVER_VERSION}
+import org.apache.kyuubi.operation.{OperationHandle, TClientTestUtils}
+import org.apache.kyuubi.service.TFrontendService.FeServiceServerContext
 import org.apache.kyuubi.session.{AbstractSession, SessionHandle}
 
 class TFrontendServiceSuite extends KyuubiFunSuite {
@@ -67,15 +67,15 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
     tFetchResultsReq.setOrientation(TFetchOrientation.FETCH_NEXT)
     tFetchResultsReq.setMaxRows(10)
     val resp = client.FetchResults(tFetchResultsReq)
-    val expected = handle.getOperationType.toString
+    val expected = "NoopOperation"
     val actual = resp.getResults.getColumns.get(0).getStringVal.getValues.get(0)
     assert(actual === expected)
     tFetchResultsReq.setFetchType(1)
     val resp1 = client.FetchResults(tFetchResultsReq)
     assert(resp1.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
     assert(resp1.getResults.getColumnCount === 0)
-    val invalidHandle =
-      OperationHandle(OperationType.getOperationType(handle.getOperationType), SERVER_VERSION)
+    val invalidHandle = OperationHandle().toTOperationHandle
+    invalidHandle.setOperationType(TOperationType.EXECUTE_STATEMENT)
     tFetchResultsReq.setOperationHandle(invalidHandle)
     val errResp = client.FetchResults(tFetchResultsReq)
     errResp.getStatus
@@ -203,7 +203,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(opHandle.getOperationType === TOperationType.GET_CATALOGS)
       assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       checkOperationResult(client, opHandle)
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
 
       val resp1 = client.GetCatalogs(req)
       assert(resp1.getOperationHandle === null)
@@ -222,7 +222,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       checkOperationResult(client, opHandle)
 
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
       val resp1 = client.GetSchemas(req)
       assert(resp1.getOperationHandle === null)
       assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -240,7 +240,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       checkOperationResult(client, opHandle)
 
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
       val resp1 = client.GetTables(req)
       assert(resp1.getOperationHandle === null)
       assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -258,7 +258,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       checkOperationResult(client, opHandle)
 
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
       val resp1 = client.GetTableTypes(req)
       assert(resp1.getOperationHandle === null)
       assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -276,7 +276,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       checkOperationResult(client, opHandle)
 
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
       val resp1 = client.GetColumns(req)
       assert(resp1.getOperationHandle === null)
       assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -294,7 +294,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       checkOperationResult(client, opHandle)
 
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
       val resp1 = client.GetFunctions(req)
       assert(resp1.getOperationHandle === null)
       assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -313,7 +313,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       checkOperationResult(client, opHandle)
 
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
       val resp1 = client.GetTypeInfo(req)
       assert(resp1.getOperationHandle === null)
       assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -326,10 +326,17 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
     withSessionHandle { (client, handle) =>
       val req = new TGetPrimaryKeysReq(handle)
       val resp = client.GetPrimaryKeys(req)
-      assert(resp.getOperationHandle === null)
-      assert(resp.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
-      assert(resp.getStatus.getSqlState === "0A000")
-      assert(resp.getStatus.getErrorMessage startsWith "feature not supported")
+      val opHandle = resp.getOperationHandle
+      assert(opHandle.getOperationType === TOperationType.GET_FUNCTIONS)
+      assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
+      checkOperationResult(client, opHandle)
+
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
+      val resp1 = client.GetPrimaryKeys(req)
+      assert(resp1.getOperationHandle === null)
+      assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
+      assert(resp1.getStatus.getSqlState === null)
+      assert(resp1.getStatus.getErrorMessage startsWith "Invalid SessionHandle")
     }
   }
 
@@ -337,17 +344,38 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
     withSessionHandle { (client, handle) =>
       val req = new TGetCrossReferenceReq(handle)
       val resp = client.GetCrossReference(req)
-      assert(resp.getOperationHandle === null)
-      assert(resp.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
-      assert(resp.getStatus.getSqlState === "0A000")
-      assert(resp.getStatus.getErrorMessage startsWith "feature not supported")
+      val opHandle = resp.getOperationHandle
+      assert(opHandle.getOperationType === TOperationType.GET_FUNCTIONS)
+      assert(resp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
+      checkOperationResult(client, opHandle)
+
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
+      val resp1 = client.GetCrossReference(req)
+      assert(resp1.getOperationHandle === null)
+      assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
+      assert(resp1.getStatus.getSqlState === null)
+      assert(resp1.getStatus.getErrorMessage startsWith "Invalid SessionHandle")
+    }
+  }
+
+  test("get query id") {
+    withSessionHandle { (client, handle) =>
+      val req = new TExecuteStatementReq()
+      req.setStatement("select 1")
+      req.setSessionHandle(handle)
+      req.setRunAsync(false)
+      val resp = client.ExecuteStatement(req)
+      val opHandle = resp.getOperationHandle
+      val req1 = new TGetQueryIdReq(opHandle)
+      val resp1 = client.GetQueryId(req1)
+      assert(resp1.getQueryId === "noop_query_id")
     }
   }
 
   test("get operation status") {
     withSessionHandle { (client, handle) =>
-      val opHandle =
-        OperationHandle(OperationType.EXECUTE_STATEMENT, SERVER_VERSION)
+      val opHandle = OperationHandle().toTOperationHandle
+      opHandle.setOperationType(TOperationType.GET_TABLES)
       val req = new TGetOperationStatusReq(opHandle)
       val resp = client.GetOperationStatus(req)
       assert(resp.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -368,7 +396,8 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       val resp4 = client.GetOperationStatus(req2)
       assert(resp4.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
       assert(resp4.getOperationState === TOperationState.ERROR_STATE)
-      assert(resp4.getErrorMessage === "noop operation err")
+      assert(resp4.getErrorMessage startsWith "org.apache.kyuubi.KyuubiSQLException:" +
+        " noop operation err\n\tat org.apache.kyuubi.KyuubiSQLException")
     }
   }
 
@@ -388,7 +417,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       val opHandle1 = resp1.getOperationHandle
       checkOperationResult(client, opHandle1)
 
-      req.setSessionHandle(SessionHandle(SERVER_VERSION).toTSessionHandle)
+      req.setSessionHandle(SessionHandle().toTSessionHandle)
       val resp2 = client.ExecuteStatement(req)
       assert(resp2.getOperationHandle === null)
       assert(resp2.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
@@ -399,8 +428,8 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
 
   test("cancel operation") {
     withSessionHandle { (client, handle) =>
-      val opHandle =
-        OperationHandle(OperationType.EXECUTE_STATEMENT, SERVER_VERSION)
+      val opHandle = OperationHandle().toTOperationHandle
+      opHandle.setOperationType(TOperationType.EXECUTE_STATEMENT)
       val req = new TCancelOperationReq()
       req.setOperationHandle(opHandle)
       val resp = client.CancelOperation(req)
@@ -419,8 +448,8 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
 
   test("close operation") {
     withSessionHandle { (client, handle) =>
-      val opHandle =
-        OperationHandle(OperationType.EXECUTE_STATEMENT, SERVER_VERSION)
+      val opHandle = OperationHandle().toTOperationHandle
+      opHandle.setOperationType(TOperationType.EXECUTE_STATEMENT)
       val req = new TCloseOperationReq()
       req.setOperationHandle(opHandle)
       val resp = client.CloseOperation(req)
@@ -448,7 +477,9 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       assert(resp.getSchema.getColumns.get(0).getComment === "comment")
       assert(resp.getSchema.getColumns.get(0).getPosition === 0)
 
-      req1.setOperationHandle(OperationHandle(OperationType.EXECUTE_STATEMENT, SERVER_VERSION))
+      val handle1 = OperationHandle().toTOperationHandle
+      handle1.setOperationType(TOperationType.GET_SCHEMAS)
+      req1.setOperationHandle(handle1)
       val resp1 = client.GetResultSetMetadata(req1)
       assert(resp1.getStatus.getStatusCode === TStatusCode.ERROR_STATUS)
       assert(resp1.getStatus.getSqlState === null)
@@ -456,7 +487,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
     }
   }
 
-  test("Delegation token is not supported") {
+  test("Delegation token is supported by default") {
     withSessionHandle { (client, handle) =>
       val tGetDelegationTokenReq = new TGetDelegationTokenReq()
       tGetDelegationTokenReq.setSessionHandle(handle)
@@ -478,8 +509,7 @@ class TFrontendServiceSuite extends KyuubiFunSuite {
       tRenewDelegationTokenReq.setSessionHandle(handle)
       tRenewDelegationTokenReq.setDelegationToken("")
       val tRenewDelegationTokenResp = client.RenewDelegationToken(tRenewDelegationTokenReq)
-      assert(tRenewDelegationTokenResp.getStatus.getErrorMessage ===
-        "Delegation token is not supported")
+      assert(tRenewDelegationTokenResp.getStatus.getStatusCode === TStatusCode.SUCCESS_STATUS)
     }
   }
 
