@@ -17,31 +17,27 @@
 
 package org.apache.kyuubi.engine
 
-import java.util.UUID
 import java.util.concurrent.TimeUnit
 
 import scala.util.Random
 
 import com.codahale.metrics.MetricRegistry
 import com.google.common.annotations.VisibleForTesting
-import org.apache.curator.framework.CuratorFramework
-import org.apache.curator.framework.recipes.locks.InterProcessSemaphoreMutex
-import org.apache.curator.utils.ZKPaths
 import org.apache.hadoop.security.UserGroupInformation
 
 import org.apache.kyuubi.{KYUUBI_VERSION, KyuubiSQLException, Logging, Utils}
 import org.apache.kyuubi.config.KyuubiConf
 import org.apache.kyuubi.config.KyuubiConf._
 import org.apache.kyuubi.config.KyuubiReservedKeys.KYUUBI_ENGINE_SUBMIT_TIME_KEY
-import org.apache.kyuubi.engine.EngineType.{EngineType, FLINK_SQL, SPARK_SQL, TRINO}
+import org.apache.kyuubi.engine.EngineType.{EngineType, FLINK_SQL, HIVE_SQL, JDBC, SPARK_SQL, TRINO}
 import org.apache.kyuubi.engine.ShareLevel.{CONNECTION, GROUP, SERVER, ShareLevel}
 import org.apache.kyuubi.engine.flink.FlinkProcessBuilder
+import org.apache.kyuubi.engine.hive.HiveProcessBuilder
+import org.apache.kyuubi.engine.jdbc.JdbcProcessBuilder
 import org.apache.kyuubi.engine.spark.SparkProcessBuilder
 import org.apache.kyuubi.engine.trino.TrinoProcessBuilder
-import org.apache.kyuubi.ha.HighAvailabilityConf.HA_ZK_ENGINE_REF_ID
-import org.apache.kyuubi.ha.HighAvailabilityConf.HA_ZK_NAMESPACE
-import org.apache.kyuubi.ha.client.ServiceDiscovery.getEngineByRefId
-import org.apache.kyuubi.ha.client.ServiceDiscovery.getServerHost
+import org.apache.kyuubi.ha.HighAvailabilityConf.{HA_ENGINE_REF_ID, HA_NAMESPACE}
+import org.apache.kyuubi.ha.client.{DiscoveryClient, DiscoveryPaths}
 import org.apache.kyuubi.metrics.MetricsConstants.{ENGINE_FAIL, ENGINE_TIMEOUT, ENGINE_TOTAL}
 import org.apache.kyuubi.metrics.MetricsSystem
 import org.apache.kyuubi.operation.log.OperationLog
@@ -56,10 +52,11 @@ import org.apache.kyuubi.operation.log.OperationLog
 private[kyuubi] class EngineRef(
     conf: KyuubiConf,
     user: String,
-    engineRefId: String = UUID.randomUUID().toString)
+    engineRefId: String,
+    engineManager: KyuubiApplicationManager)
   extends Logging {
   // The corresponding ServerSpace where the engine belongs to
-  private val serverSpace: String = conf.get(HA_ZK_NAMESPACE)
+  private val serverSpace: String = conf.get(HA_NAMESPACE)
 
   private val timeout: Long = conf.get(ENGINE_INIT_TIMEOUT)
 
@@ -79,6 +76,8 @@ private[kyuubi] class EngineRef(
   // a small amount of time for timeout
   private val LOCK_TIMEOUT_SPAN_FACTOR = if (Utils.isTesting) 0.5 else 0.1
 
+  private var builder: ProcBuilder = _
+
   @VisibleForTesting
   private[kyuubi] val subdomain: String = conf.get(ENGINE_SHARE_LEVEL_SUBDOMAIN) match {
     case Some(_subdomain) => _subdomain
@@ -126,20 +125,20 @@ private[kyuubi] class EngineRef(
    * The EngineSpace used to expose itself to the KyuubiServers in `serverSpace`
    *
    * For `CONNECTION` share level:
-   *   /`serverSpace_CONNECTION_engineType`/`user`/`engineRefId`
+   *   /`serverSpace_version_CONNECTION_engineType`/`user`/`engineRefId`
    * For `USER` share level:
-   *   /`serverSpace_USER_engineType`/`user`[/`subdomain`]
+   *   /`serverSpace_version_USER_engineType`/`user`[/`subdomain`]
    * For `GROUP` share level:
-   *   /`serverSpace_GROUP_engineType`/`primary group name`[/`subdomain`]
+   *   /`serverSpace_version_GROUP_engineType`/`primary group name`[/`subdomain`]
    * For `SERVER` share level:
-   *   /`serverSpace_SERVER_engineType`/`kyuubi server user`[/`subdomain`]
+   *   /`serverSpace_version_SERVER_engineType`/`kyuubi server user`[/`subdomain`]
    */
   @VisibleForTesting
   private[kyuubi] lazy val engineSpace: String = {
     val commonParent = s"${serverSpace}_${KYUUBI_VERSION}_${shareLevel}_$engineType"
     shareLevel match {
-      case CONNECTION => ZKPaths.makePath(commonParent, appUser, engineRefId)
-      case _ => ZKPaths.makePath(commonParent, appUser, subdomain)
+      case CONNECTION => DiscoveryPaths.makePath(commonParent, appUser, Array(engineRefId))
+      case _ => DiscoveryPaths.makePath(commonParent, appUser, Array(subdomain))
     }
   }
 
@@ -147,95 +146,51 @@ private[kyuubi] class EngineRef(
    * The distributed lock path used to ensure only once engine being created for non-CONNECTION
    * share level.
    */
-  private def tryWithLock[T](zkClient: CuratorFramework)(f: => T): T = shareLevel match {
-    case CONNECTION => f
-    case _ =>
-      val lockPath =
-        ZKPaths.makePath(s"${serverSpace}_$shareLevel", "lock", appUser, subdomain)
-      var lock: InterProcessSemaphoreMutex = null
-      try {
-        try {
-          lock = new InterProcessSemaphoreMutex(zkClient, lockPath)
-          // Acquire a lease. If no leases are available, this method blocks until either the
-          // maximum number of leases is increased or another client/process closes a lease.
-          //
-          // Here, we should throw exception if timeout during acquiring lock.
-          // Let's say we have three clients with same request lock to two kyuubi server instances.
-          //
-          //  client A  --->  kyuubi X  -- first acquired  \
-          //  client B  --->  kyuubi X  -- second acquired --  zookeeper
-          //  client C  --->  kyuubi Y  -- third acquired  /
-          //
-          // The first client A acqiured the lock then B and C are blocked until A release the lock,
-          // with the A created engine state:
-          //   - SUCCESS
-          //     B acquired the lock then get engine ref and release the lock.
-          //     C acquired the lock then get engine ref and release the lock.
-          //   - FAILED or TIMEOUT
-          //     B acquired the lock then try to create engine again if not timeout.
-          //     C should be timeout and throw exception back to client. This fast fail
-          //     to avoid client too long to waiting in concurrent.
-
-          // Return false means we are timeout
-          val acquired = lock.acquire(
-            timeout + (LOCK_TIMEOUT_SPAN_FACTOR * timeout).toLong,
-            TimeUnit.MILLISECONDS)
-          if (!acquired) {
-            throw KyuubiSQLException(s"Timeout to lock on path [$lockPath] after " +
-              s"$timeout ms. There would be some problem that other session may " +
-              s"create engine timeout.")
-          }
-        } catch {
-          case e: KyuubiSQLException => throw e
-          case e: Exception => throw KyuubiSQLException(s"Lock failed on path [$lockPath]", e)
-        }
-        f
-      } finally {
-        try {
-          if (lock != null) {
-            lock.release()
-          }
-        } catch {
-          case _: Exception =>
-        }
-      }
-  }
+  private def tryWithLock[T](discoveryClient: DiscoveryClient)(f: => T): T =
+    shareLevel match {
+      case CONNECTION => f
+      case _ =>
+        val lockPath =
+          DiscoveryPaths.makePath(
+            s"${serverSpace}_${shareLevel}_$engineType",
+            "lock",
+            Array(appUser, subdomain))
+        discoveryClient.tryWithLock(
+          lockPath,
+          timeout + (LOCK_TIMEOUT_SPAN_FACTOR * timeout).toLong)(f)
+    }
 
   private def create(
-      zkClient: CuratorFramework,
-      extraEngineLog: Option[OperationLog]): (String, Int) = tryWithLock(zkClient) {
+      discoveryClient: DiscoveryClient,
+      extraEngineLog: Option[OperationLog]): (String, Int) = tryWithLock(discoveryClient) {
     // Get the engine address ahead if another process has succeeded
-    var engineRef = getServerHost(zkClient, engineSpace)
+    var engineRef = discoveryClient.getServerHost(engineSpace)
     if (engineRef.nonEmpty) return engineRef.get
 
-    conf.set(HA_ZK_NAMESPACE, engineSpace)
-    conf.set(HA_ZK_ENGINE_REF_ID, engineRefId)
+    conf.set(HA_NAMESPACE, engineSpace)
+    conf.set(HA_ENGINE_REF_ID, engineRefId)
     val started = System.currentTimeMillis()
     conf.set(KYUUBI_ENGINE_SUBMIT_TIME_KEY, String.valueOf(started))
-    val builder = engineType match {
+    builder = engineType match {
       case SPARK_SQL =>
         conf.setIfMissing(SparkProcessBuilder.APP_KEY, defaultEngineName)
-        // tag is a seq type with comma-separated
-        conf.set(
-          SparkProcessBuilder.TAG_KEY,
-          conf.getOption(SparkProcessBuilder.TAG_KEY).map(_ + ",").getOrElse("") + "KYUUBI")
-        new SparkProcessBuilder(appUser, conf, extraEngineLog)
+        new SparkProcessBuilder(appUser, conf, engineRefId, extraEngineLog)
       case FLINK_SQL =>
         conf.setIfMissing(FlinkProcessBuilder.APP_KEY, defaultEngineName)
-        // tag is a seq type with comma-separated
-        conf.set(
-          FlinkProcessBuilder.TAG_KEY,
-          conf.getOption(FlinkProcessBuilder.TAG_KEY).map(_ + ",").getOrElse("") + "KYUUBI")
-        conf.set(HA_ZK_NAMESPACE, engineSpace)
-        conf.set(HA_ZK_ENGINE_REF_ID, engineRefId)
-        new FlinkProcessBuilder(appUser, conf, extraEngineLog)
+        new FlinkProcessBuilder(appUser, conf, engineRefId, extraEngineLog)
       case TRINO =>
-        new TrinoProcessBuilder(appUser, conf, extraEngineLog)
+        new TrinoProcessBuilder(appUser, conf, engineRefId, extraEngineLog)
+      case HIVE_SQL =>
+        new HiveProcessBuilder(appUser, conf, engineRefId, extraEngineLog)
+      case JDBC =>
+        new JdbcProcessBuilder(appUser, conf, engineRefId, extraEngineLog)
     }
 
     MetricsSystem.tracing(_.incCount(ENGINE_TOTAL))
     try {
-      info(s"Launching engine:\n$builder")
+      val redactedCmd = builder.toString
+      info(s"Launching engine:\n$redactedCmd")
+      builder.validateConf
       val process = builder.start
       var exitValue: Option[Int] = None
       while (engineRef.isEmpty) {
@@ -251,14 +206,15 @@ private[kyuubi] class EngineRef(
           }
         }
         if (started + timeout <= System.currentTimeMillis()) {
-          val killMessage = builder.killApplication()
+          val killMessage = engineManager.killApplication(builder.clusterManager(), engineRefId)
           process.destroyForcibly()
           MetricsSystem.tracing(_.incCount(MetricRegistry.name(ENGINE_TIMEOUT, appUser)))
           throw KyuubiSQLException(
-            s"Timeout($timeout ms) to launched $engineType engine with $builder. $killMessage",
+            s"Timeout($timeout ms, you can modify ${ENGINE_INIT_TIMEOUT.key} to change it) to" +
+              s" launched $engineType engine with $redactedCmd. $killMessage",
             builder.getError)
         }
-        engineRef = getEngineByRefId(zkClient, engineSpace, engineRefId)
+        engineRef = discoveryClient.getEngineByRefId(engineSpace, engineRefId)
       }
       engineRef.get
     } finally {
@@ -271,15 +227,28 @@ private[kyuubi] class EngineRef(
   /**
    * Get the engine ref from engine space first or create a new one
    *
-   * @param zkClient the zookeeper client to get or create engine instance
+   * @param discoveryClient the zookeeper client to get or create engine instance
    * @param extraEngineLog the launch engine operation log, used to inject engine log into it
    */
   def getOrCreate(
-      zkClient: CuratorFramework,
+      discoveryClient: DiscoveryClient,
       extraEngineLog: Option[OperationLog] = None): (String, Int) = {
-    getServerHost(zkClient, engineSpace)
+    discoveryClient.getServerHost(engineSpace)
       .getOrElse {
-        create(zkClient, extraEngineLog)
+        create(discoveryClient, extraEngineLog)
       }
   }
+
+  def close(): Unit = {
+    if (shareLevel == CONNECTION && builder != null) {
+      try {
+        val clusterManager = builder.clusterManager()
+        builder.close(true)
+        engineManager.killApplication(clusterManager, engineRefId)
+      } catch {
+        case e: Exception =>
+          warn(s"Error closing engine builder, engineRefId: $engineRefId", e)
+      }
+    }
+  }
 }
