@@ -17,52 +17,65 @@
 
 package org.apache.kyuubi.server.api.v1
 
-import javax.ws.rs.{Consumes, DELETE, GET, Path, PathParam, POST, Produces, _}
+import javax.ws.rs._
 import javax.ws.rs.core.{MediaType, Response}
 
 import scala.collection.JavaConverters._
+import scala.language.implicitConversions
 import scala.util.control.NonFatal
 
-import io.swagger.v3.oas.annotations.media.Content
+import io.swagger.v3.oas.annotations.media.{ArraySchema, Content, Schema}
 import io.swagger.v3.oas.annotations.responses.ApiResponse
 import io.swagger.v3.oas.annotations.tags.Tag
 import org.apache.hive.service.rpc.thrift.{TGetInfoType, TProtocolVersion}
 
 import org.apache.kyuubi.Logging
+import org.apache.kyuubi.client.api.v1.dto
+import org.apache.kyuubi.client.api.v1.dto._
 import org.apache.kyuubi.events.KyuubiEvent
 import org.apache.kyuubi.operation.OperationHandle
 import org.apache.kyuubi.server.api.ApiRequestContext
+import org.apache.kyuubi.session.KyuubiSession
 import org.apache.kyuubi.session.SessionHandle
-import org.apache.kyuubi.session.SessionHandle.parseSessionHandle
 
 @Tag(name = "Session")
 @Produces(Array(MediaType.APPLICATION_JSON))
 private[v1] class SessionsResource extends ApiRequestContext with Logging {
+  implicit def toSessionHandle(str: String): SessionHandle = SessionHandle.fromUUID(str)
+  private def sessionManager = fe.be.sessionManager
 
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
-    description = "get all the session list hosted in SessionManager")
+      mediaType = MediaType.APPLICATION_JSON,
+      array = new ArraySchema(schema = new Schema(implementation = classOf[SessionData])))),
+    description = "get the list of all live sessions")
   @GET
-  def sessionInfoList(): SessionList = {
-    SessionList(
-      fe.be.sessionManager.getSessionList().asScala.map {
-        case (handle, session) =>
-          SessionOverview(session.user, session.ipAddress, session.createTime, handle)
-      }.toSeq)
+  def sessions(): Seq[SessionData] = {
+    sessionManager.allSessions().map { session =>
+      new SessionData(
+        session.handle.identifier.toString,
+        session.user,
+        session.ipAddress,
+        session.conf.asJava,
+        session.createTime,
+        session.lastAccessTime - session.createTime,
+        session.getNoOperationTime)
+    }.toSeq
   }
 
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[KyuubiEvent]))),
     description = "get a session event via session handle identifier")
   @GET
   @Path("{sessionHandle}")
   def sessionInfo(@PathParam("sessionHandle") sessionHandleStr: String): KyuubiEvent = {
     try {
-      fe.be.sessionManager.getSession(parseSessionHandle(sessionHandleStr)).getSessionEvent.get
+      sessionManager.getSession(sessionHandleStr)
+        .asInstanceOf[KyuubiSession].getSessionEvent.get
     } catch {
       case NonFatal(e) =>
         error(s"Invalid $sessionHandleStr", e)
@@ -73,7 +86,8 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[InfoDetail]))),
     description =
       "get a information detail via session handle identifier and a specific information type")
   @GET
@@ -83,8 +97,8 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
       @PathParam("infoType") infoType: Int): InfoDetail = {
     try {
       val info = TGetInfoType.findByValue(infoType)
-      val infoValue = fe.be.getInfo(parseSessionHandle(sessionHandleStr), info)
-      InfoDetail(info.toString, infoValue.getStringValue)
+      val infoValue = fe.be.getInfo(sessionHandleStr, info)
+      new InfoDetail(info.toString, infoValue.getStringValue)
     } catch {
       case NonFatal(e) =>
         error(s"Unrecognized GetInfoType value: $infoType", e)
@@ -95,25 +109,27 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[SessionOpenCount]))),
     description = "Get the current open session count")
   @GET
   @Path("count")
   def sessionCount(): SessionOpenCount = {
-    SessionOpenCount(fe.be.sessionManager.getOpenSessionCount)
+    new SessionOpenCount(sessionManager.getOpenSessionCount)
   }
 
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[ExecPoolStatistic]))),
     description = "Get statistic info of background executors")
   @GET
   @Path("execPool/statistic")
   def execPoolStatistic(): ExecPoolStatistic = {
-    ExecPoolStatistic(
-      fe.be.sessionManager.getExecPoolSize,
-      fe.be.sessionManager.getActiveCount)
+    new ExecPoolStatistic(
+      sessionManager.getExecPoolSize,
+      sessionManager.getActiveCount)
   }
 
   @ApiResponse(
@@ -123,13 +139,16 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
     description = "Open(create) a session")
   @POST
   @Consumes(Array(MediaType.APPLICATION_JSON))
-  def openSession(request: SessionOpenRequest): SessionHandle = {
-    fe.be.openSession(
-      TProtocolVersion.findByValue(request.protocolVersion),
-      request.user,
-      request.password,
-      request.ipAddr,
-      request.configs)
+  def openSession(request: SessionOpenRequest): dto.SessionHandle = {
+    val userName = fe.getUserName(request.getConfigs.asScala.toMap)
+    val ipAddress = fe.getIpAddress
+    val handle = fe.be.openSession(
+      TProtocolVersion.findByValue(request.getProtocolVersion),
+      userName,
+      request.getPassword,
+      ipAddress,
+      request.getConfigs.asScala.toMap)
+    new dto.SessionHandle(handle.identifier)
   }
 
   @ApiResponse(
@@ -140,14 +159,15 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @DELETE
   @Path("{sessionHandle}")
   def closeSession(@PathParam("sessionHandle") sessionHandleStr: String): Response = {
-    fe.be.closeSession(parseSessionHandle(sessionHandleStr))
+    fe.be.closeSession(sessionHandleStr)
     Response.ok().build()
   }
 
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with EXECUTE_STATEMENT type")
   @POST
   @Path("{sessionHandle}/operations/statement")
@@ -156,11 +176,11 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
       request: StatementRequest): OperationHandle = {
     try {
       fe.be.executeStatement(
-        parseSessionHandle(sessionHandleStr),
-        request.statement,
+        sessionHandleStr,
+        request.getStatement,
         Map.empty,
-        request.runAsync,
-        request.queryTimeout)
+        request.isRunAsync,
+        request.getQueryTimeout)
     } catch {
       case NonFatal(e) =>
         val errorMsg = "Error executing statement"
@@ -172,13 +192,14 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with GET_TYPE_INFO type")
   @POST
   @Path("{sessionHandle}/operations/typeInfo")
   def getTypeInfo(@PathParam("sessionHandle") sessionHandleStr: String): OperationHandle = {
     try {
-      fe.be.getTypeInfo(parseSessionHandle(sessionHandleStr))
+      fe.be.getTypeInfo(sessionHandleStr)
     } catch {
       case NonFatal(e) =>
         val errorMsg = "Error getting type information"
@@ -190,13 +211,14 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with GET_CATALOGS type")
   @POST
   @Path("{sessionHandle}/operations/catalogs")
   def getCatalogs(@PathParam("sessionHandle") sessionHandleStr: String): OperationHandle = {
     try {
-      fe.be.getCatalogs(parseSessionHandle(sessionHandleStr))
+      fe.be.getCatalogs(sessionHandleStr)
     } catch {
       case NonFatal(e) =>
         val errorMsg = "Error getting catalogs"
@@ -208,7 +230,8 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with GET_SCHEMAS type")
   @POST
   @Path("{sessionHandle}/operations/schemas")
@@ -216,11 +239,10 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
       @PathParam("sessionHandle") sessionHandleStr: String,
       request: GetSchemasRequest): OperationHandle = {
     try {
-      val sessionHandle = parseSessionHandle(sessionHandleStr)
       val operationHandle = fe.be.getSchemas(
-        sessionHandle,
-        request.catalogName,
-        request.schemaName)
+        sessionHandleStr,
+        request.getCatalogName,
+        request.getSchemaName)
       operationHandle
     } catch {
       case NonFatal(e) =>
@@ -233,7 +255,8 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with GET_TABLES type")
   @POST
   @Path("{sessionHandle}/operations/tables")
@@ -242,11 +265,11 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
       request: GetTablesRequest): OperationHandle = {
     try {
       fe.be.getTables(
-        parseSessionHandle(sessionHandleStr),
-        request.catalogName,
-        request.schemaName,
-        request.tableName,
-        request.tableTypes)
+        sessionHandleStr,
+        request.getCatalogName,
+        request.getSchemaName,
+        request.getTableName,
+        request.getTableTypes)
     } catch {
       case NonFatal(e) =>
         val errorMsg = "Error getting tables"
@@ -258,13 +281,14 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with GET_TABLE_TYPES type")
   @POST
   @Path("{sessionHandle}/operations/tableTypes")
   def getTableTypes(@PathParam("sessionHandle") sessionHandleStr: String): OperationHandle = {
     try {
-      fe.be.getTableTypes(parseSessionHandle(sessionHandleStr))
+      fe.be.getTableTypes(sessionHandleStr)
     } catch {
       case NonFatal(e) =>
         val errorMsg = "Error getting table types"
@@ -276,7 +300,8 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with GET_COLUMNS type")
   @POST
   @Path("{sessionHandle}/operations/columns")
@@ -285,11 +310,11 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
       request: GetColumnsRequest): OperationHandle = {
     try {
       fe.be.getColumns(
-        parseSessionHandle(sessionHandleStr),
-        request.catalogName,
-        request.schemaName,
-        request.tableName,
-        request.columnName)
+        sessionHandleStr,
+        request.getCatalogName,
+        request.getSchemaName,
+        request.getTableName,
+        request.getColumnName)
     } catch {
       case NonFatal(e) =>
         val errorMsg = "Error getting columns"
@@ -301,7 +326,8 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
   @ApiResponse(
     responseCode = "200",
     content = Array(new Content(
-      mediaType = MediaType.APPLICATION_JSON)),
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
     description = "Create an operation with GET_FUNCTIONS type")
   @POST
   @Path("{sessionHandle}/operations/functions")
@@ -310,10 +336,10 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
       request: GetFunctionsRequest): OperationHandle = {
     try {
       fe.be.getFunctions(
-        parseSessionHandle(sessionHandleStr),
-        request.catalogName,
-        request.schemaName,
-        request.functionName)
+        sessionHandleStr,
+        request.getCatalogName,
+        request.getSchemaName,
+        request.getFunctionName)
     } catch {
       case NonFatal(e) =>
         val errorMsg = "Error getting functions"
@@ -321,4 +347,57 @@ private[v1] class SessionsResource extends ApiRequestContext with Logging {
         throw new NotFoundException(errorMsg)
     }
   }
+
+  @ApiResponse(
+    responseCode = "200",
+    content = Array(new Content(
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
+    description = "Create an operation with GET_FUNCTIONS type")
+  @POST
+  @Path("{sessionHandle}/operations/primaryKeys")
+  def getPrimaryKeys(
+      @PathParam("sessionHandle") sessionHandleStr: String,
+      request: GetPrimaryKeysRequest): OperationHandle = {
+    try {
+      fe.be.getPrimaryKeys(
+        sessionHandleStr,
+        request.getCatalogName,
+        request.getSchemaName,
+        request.getTableName)
+    } catch {
+      case NonFatal(e) =>
+        val errorMsg = "Error getting primary keys"
+        error(errorMsg, e)
+        throw new NotFoundException(errorMsg)
+    }
+  }
+
+  @ApiResponse(
+    responseCode = "200",
+    content = Array(new Content(
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[OperationHandle]))),
+    description = "Create an operation with GET_FUNCTIONS type")
+  @POST
+  @Path("{sessionHandle}/operations/crossReference")
+  def getCrossReference(
+      @PathParam("sessionHandle") sessionHandleStr: String,
+      request: GetCrossReferenceRequest): OperationHandle = {
+    try {
+      fe.be.getCrossReference(
+        sessionHandleStr,
+        request.getPrimaryCatalog,
+        request.getPrimarySchema,
+        request.getPrimaryTable,
+        request.getForeignCatalog,
+        request.getForeignSchema,
+        request.getForeignTable)
+    } catch {
+      case NonFatal(e) =>
+        val errorMsg = "Error getting cross reference"
+        error(errorMsg, e)
+        throw new NotFoundException(errorMsg)
+    }
+  }
 }
